#include "common.h"
#include "gpio_pwm_carship.h"
#include "gpio_pwm_car0102.h"
#include "gpio_pwm_ptz.h"
#include "gpio_pwm_tank0202.h"
#include "ip.h"
#include "opensh.h"
#include "INA226.h"
#include "heat.h"
#include "mqtt.h"
#include "warm.h"
#include "fileopen.h"
#include "device_change.h"
#include "judg.h"

struct mosquitto *mosq;

char*  TOPIC = NULL;
char*  TOPIC2 = NULL;
char*  TOPIC3 = NULL;

char*  TOPIC_BACK = NULL;
char*  TOPIC2_BACK = NULL;
char*  TOPIC3_BACK = NULL;

int gPwmCount = 0;  // 计数
int gmessage_type=10086;

int message;//存放gmessage_type的值发送心跳，2，3消息为1；

uint16_t AppExit_pin_pwm=0;//判断坦克或者车的退出

int rc=0;//判断mqtt是否成功创建
//ThreadPool *pool;//线程池线程函数

unsigned char modeTemp=0;
unsigned char typeTemp=0;
unsigned char pinTemp=0;
unsigned char valTemp=0;
unsigned char gvalt[4];//存放mqtt接收的tpye，mode等

char  *glat=NULL;//加入gps后删除，心跳预留，不更改
char  *glon=NULL;//加入gps后删除，心跳预留，不更改

time_t gStart;//时间戳

int mqtt_init() {
 // 初始化 mosquitto 库
    mosquitto_lib_init();  

    mosq = mosquitto_new(NULL, true, NULL);
    if (!mosq) {
        my_zlog_fatal("Failed to create Mosquitto client");
        return -1;
    }

    mosquitto_int_option(mosq, MOSQ_OPT_PROTOCOL_VERSION, MQTT_PROTOCOL_V5);

    return 0;
}

void on_connect(struct mosquitto *mosq, void *obj, int rc) {//回调函数
    if (rc == 0) {
        my_zlog_debug("Connected to broker");
        topic_middle_value();
        mosquitto_subscribe(mosq, NULL, TOPIC, 0);
     //mosquitto_subscribe(mosq, NULL, TOPIC_BACK, 0);
    } else {
        fprintf(stderr, "Connection failed with code %d\n", rc);
        my_zlog_fatal("Connection failed");
    }
}

void mqtt_wirte(){//心跳格式，每5s一次心跳

    float voltage = INA226_readBusVoltage(sda_pin, scl_pin);
    float current = INA226_readCurrent(sda_pin, scl_pin);
    char voltage_str[20];  // 足够存储转换后的字符串的缓冲区
    sprintf(voltage_str, "%.2f", voltage); 
    char current_str[20];  // 足够存储转换后的字符串的缓冲区
    sprintf(current_str, "%.2f", current); 

    if(AppExit_pin_pwm != 102 || AppExit_pin_pwm != 202 || AppExit_pin_pwm != 301 ||AppExit_pin_pwm != 401) {   
        alarm_control(voltage);//判断电压警报
    }

    /*读取程序版本号*/
    char *version_num = program_version();

    if(heat_tem() !=0 ) {//获取CPU温度
    
    }
    message=1;

    topic_middle_value();

    cJSON *root = cJSON_CreateObject();
    cJSON *body = cJSON_CreateObject();
    cJSON *head = cJSON_CreateObject();

    cJSON_AddStringToObject(body, "ip", ip_address);//发送设备id
    cJSON_AddStringToObject(body, "ID", TOPIC);//发送设备号
    cJSON_AddStringToObject(body, "V:", voltage_str);//心跳发送电压
    cJSON_AddStringToObject(body, "I:", current_str);//心跳发送电流
    cJSON_AddStringToObject(body,"Tem:",temperature);//发送温度
    cJSON_AddStringToObject(body, "N", glat);//gps
    cJSON_AddStringToObject(body, "E", glon);//gps
    cJSON_AddStringToObject(body, "version", version_num);//gps

    cJSON_AddNumberToObject(head, "message_type",message);

    cJSON_AddItemToObject(root, "body", body);
    cJSON_AddItemToObject(root, "head",head);

    char *payload = cJSON_PrintUnformatted(root);
    //printf("%s\n",payload);
    my_zlog_debug("%s",payload);
    mosquitto_publish(mosq, NULL, TOPIC2, strlen(payload), payload, 0, false); 
    mosquitto_publish(mosq, NULL, TOPIC3, strlen(payload), payload, 0, false); 
    
    cJSON_Delete(root);  // 释放 cJSON 对象

    return ;
}

void message_3(cJSON *body,cJSON *pwm_ctrl){//message_type为3，控制pwm  
    cJSON *mode = cJSON_GetObjectItem(pwm_ctrl, "mode");   //mode=1 速度，mode=2 转向(unsigned char)
    cJSON *type = cJSON_GetObjectItem(pwm_ctrl, "type");
    cJSON *val =   cJSON_GetObjectItem(pwm_ctrl, "val");  //val为pwm的值 0~100(unsigned char)(unsigned char)
    
    modeTemp= mode->valueint;
    typeTemp=type->valueint;
    valTemp= val->valueint;

    if(typeTemp==3){ 
        valTemp= val->valueint;
    }

    gvalt[0]=typeTemp;
    gvalt[1]=modeTemp;
    gvalt[2]=valTemp;
    gPwmCount = 0;

    my_zlog_debug("typeTemp:%d",gvalt[0]);
    my_zlog_debug("modeTemp:%d",gvalt[1]);
    my_zlog_debug("valTemp:%d",gvalt[2]);

    if(AppExit_pin_pwm == 101) speed_change(gvalt) ;
    if(AppExit_pin_pwm == 102) car0102_speed_change(gvalt) ;
    if(AppExit_pin_pwm == 202) tank0202_change(gvalt);
    if(AppExit_pin_pwm == 301) ship_speed_change(gvalt) ;
    if(AppExit_pin_pwm == 401) PTZ_pwm_change(gvalt);
        
}

void message_4(cJSON *body, cJSON *pin_setctrl){//message 为4时候
    cJSON *pin = cJSON_GetObjectItem(pin_setctrl, "pin");
    cJSON *val = cJSON_GetObjectItem(pin_setctrl, "val");  //val为pwm的值 0~100(unsigned char)(unsigned char)
    
    pinTemp=pin->valueint;
    valTemp= val->valueint;

    if(valTemp>=1){
        valTemp=1;
    }

    gPwmCount=0;
    gvalt[0]=0;
    gvalt[1]=pinTemp;
    gvalt[2]=valTemp;

    my_zlog_debug("pinTemp:%d",gvalt[1]);
    my_zlog_debug("valTemp:%d",gvalt[2]);

    pin_value(gvalt[1],gvalt[2]);
    pwm_value(gvalt[1],gvalt[2]);
}

int device_message(cJSON *json){//接收到的控制设备的mqtt消息
    
    cJSON *head = cJSON_GetObjectItem(json, "head");
    cJSON *message_type = cJSON_GetObjectItem(head, "message_type");

    if (cJSON_IsObject(head)&&cJSON_IsNumber(message_type)) {;
            
        cJSON *body = cJSON_GetObjectItem(json, "body");//提取 body数据段
        if (!cJSON_IsObject(body)) {
            return 1;
        }

        cJSON *pwm_ctrl = cJSON_GetObjectItem(body, "pwm_ctrl");
        cJSON *pin_setctrl = cJSON_GetObjectItem(body, "pin_setctrl");
        if(pwm_ctrl==NULL||pin_setctrl==NULL){ 
            return 2;
        }

        gmessage_type=message_type->valueint;
        my_zlog_debug("message_type: %d",message_type->valueint);
        switch(gmessage_type){
            case 1:
                system("sudo reboot");//重启香橙派
                my_zlog_debug("重启成功");
                break;
            case 2:
                refresh_cam();
                my_zlog_debug("刷新成功");
                break;
            case 3:
                message_3(body,pwm_ctrl);
                my_zlog_debug("进入pwm控制");
                break;
            case 4:
                message_4(body,pin_setctrl);
                my_zlog_debug("进入引脚控制");
                break;
            default:
                break;
        }
    }else return 3;     
 
    return 0;
}

int download_message(cJSON *root){//接收的下载的mqtt消息
   cJSON *down = cJSON_GetObjectItem(root, "down");
    if (cJSON_IsObject(down)) {
        // 获取 site 对象
        cJSON *site = cJSON_GetObjectItem(down, "site");
        if (cJSON_IsObject(site)) {
            // 获取 site 中的 name 和 url
            cJSON *name = cJSON_GetObjectItem(site, "name");
            cJSON *url = cJSON_GetObjectItem(site, "url");
            if (cJSON_IsString(name) && cJSON_IsString(url)) {
                my_zlog_debug("Site Name: %s", name->valuestring);
                my_zlog_debug("Site URL: %s", url->valuestring);
            }
        }

        // 获取 type 对象
        cJSON *type = cJSON_GetObjectItem(down, "type");
        if (cJSON_IsObject(type)) {
            // 获取 type 中的 suf、dir 和 sername
            cJSON *down_suf = cJSON_GetObjectItem(type, "suf");
            //cJSON *down_dir = cJSON_GetObjectItem(type, "dir");&& cJSON_IsString(down_dir)
            cJSON *down_sername = cJSON_GetObjectItem(type, "sername");
            if (cJSON_IsString(down_suf)  && cJSON_IsString(down_sername)) {
                my_zlog_debug("Type Suf: %s", down_suf->valuestring);
                //my_zlog_debug("Type Dir: %s", down_dir->valuestring);
                my_zlog_debug("Type Sername: %s", down_sername->valuestring);
            }
        }
    }else  return -1;

    return 0;
}


int device_mqttchange_name(cJSON *device_id){
    cJSON *id = cJSON_GetObjectItem(device_id, "id");// 提取ID   
    cJSON *date = cJSON_GetObjectItem(device_id, "date");// 提取日期
    char *device_change_id=id->valuestring;
    char *device_change_date=date->valuestring;
    device_changename_back(device_change_date,device_change_id);
    // 输出提取的数据
    my_zlog_debug("设备ID: %s", device_change_id);
    my_zlog_debug("日期: %s", device_change_date);

    return 0;
}

void on_message(struct mosquitto *mosq, void *obj, const struct mosquitto_message *message) {//消息回环函数  
    time_t end = time(NULL);

    if(gStart+5>end){
        //printf("正在处理过期消息\n");
        my_zlog_warn("正在处理过期消息");
        return;     
    }

    // 确保消息有效
    if (message->payload && message->payloadlen) { // 将消息内容转换为字符串
        char *payload_str = (char *)malloc(message->payloadlen + 1);
        if (!payload_str) {
            return;
        } 

        memcpy(payload_str, message->payload, message->payloadlen);
        payload_str[message->payloadlen] = '\0';

        // 解析 JSON
        cJSON *json = cJSON_Parse(payload_str);
        if (json == NULL) {
            //fprintf(stderr, "Error before: [%s]\n", cJSON_GetErrorPtr());
            my_zlog_error("Error before: [%s]", cJSON_GetErrorPtr());
        }  else if(cJSON_GetObjectItem(json, "device_id") != NULL) {
            cJSON *device_id=cJSON_GetObjectItem(json, "device_id");
            device_mqttchange_name(device_id); 
            my_zlog_debug("接收到设备号更改");
        } else {// 提取 head 对象
            if(download_message(json) != 1) my_zlog_debug("接收到dowwnload下载");
            if(device_message(json) != 0) my_zlog_debug("没有接收到控制信息，代码编号：%d",device_message(json));                       
        }
        // 释放 payload 字符串
        free(payload_str); 
        cJSON_Delete(json); 
    }
}


void mqtt_create(struct mosquitto *mosq) {//创建mqtt客服端
    mosquitto_reconnect_delay_set(mosq, 2, 10, true);
       // 设置连接和消息回调
    mosquitto_connect_callback_set(mosq, on_connect);
    mosquitto_message_callback_set(mosq, on_message);
    //设置用户名和密码
    mosquitto_username_pw_set(mosq, USERNAME, PASSWORD);
    // 创建 mosquitto 客户端
    rc = mosquitto_connect(mosq, BROKER_ADDRESS, BROKER_PORT, 60);
    if (rc != MOSQ_ERR_SUCCESS) {                             
        //fprintf(stderr, "Failed to connect to broker: %s\n", mosquitto_strerror(rc));
        my_zlog_warn("Failed to connect to broker: %s", mosquitto_strerror(rc));
        mosquitto_destroy(mosq);
        //return EXIT_FAILURE;
    }
}    

void mqtt_cycle(struct mosquitto *mosq){
    mosquitto_loop_forever(mosq, -1, 1); // -1 表示无限等待，1 表示处理消息的最大数量
}

void mqtt_clean(struct mosquitto *mosq){
      // 清理
    mosquitto_destroy(mosq);
    mosquitto_lib_cleanup();
    my_zlog_notice("清理成功");
}