#include <math.h>
#include "common.h"
#include "INA226.h"
float currentLSB ;

int sda_pin=0; // GPIO0
int scl_pin=1; // GPIO1
// 软件 I2C 延时
void i2c_delay() {
    delayMicroseconds(5);
}

// I2C 起始信号
void i2c_start(int sda_pin, int scl_pin) {
    digitalWrite(sda_pin, HIGH);
    digitalWrite(scl_pin, HIGH);
    i2c_delay();
    digitalWrite(sda_pin, LOW);
    i2c_delay();
    digitalWrite(scl_pin, LOW);
    i2c_delay();
}

// I2C 停止信号
void i2c_stop(int sda_pin, int scl_pin) {
    digitalWrite(sda_pin, LOW);
    digitalWrite(scl_pin, LOW);
    i2c_delay();
    digitalWrite(scl_pin, HIGH);
    i2c_delay();
    digitalWrite(sda_pin, HIGH);
    i2c_delay();
}

// I2C 发送一个字节
void i2c_write_byte(int sda_pin, int scl_pin, uint8_t data) {
    for (int i = 7; i >= 0; i--) {
        digitalWrite(sda_pin, (data >> i) & 0x01);
        i2c_delay();
        digitalWrite(scl_pin, HIGH);
        i2c_delay();
        digitalWrite(scl_pin, LOW);
        i2c_delay();
    }
    // 接收 ACK
    pinMode(sda_pin, INPUT);
    digitalWrite(scl_pin, HIGH);
    i2c_delay();
    digitalWrite(scl_pin, LOW);
    pinMode(sda_pin, OUTPUT);
}

// I2C 读取一个字节
uint8_t i2c_read_byte(int sda_pin, int scl_pin, bool ack) {
    uint8_t data = 0;
    pinMode(sda_pin, INPUT);
    for (int i = 7; i >= 0; i--) {
        digitalWrite(scl_pin, HIGH);
        i2c_delay();
        data |= (digitalRead(sda_pin) << i);
        digitalWrite(scl_pin, LOW);
        i2c_delay();
    }
    // 发送 ACK/NACK
    pinMode(sda_pin, OUTPUT);
    digitalWrite(sda_pin, ack ? LOW : HIGH);
    digitalWrite(scl_pin, HIGH);
    i2c_delay();
    digitalWrite(scl_pin, LOW);
    pinMode(sda_pin, INPUT);
    return data;
}

// 初始化 INA226
bool INA226_begin(int sda_pin, int scl_pin) {
    pinMode(sda_pin, OUTPUT);
    pinMode(scl_pin, OUTPUT);
    digitalWrite(sda_pin, HIGH);
    digitalWrite(scl_pin, HIGH);
    return true;
}

// 配置 INA226
bool INA226_configure(int sda_pin, int scl_pin) {
    uint16_t config = 0x4127; // 默认配置：16 次平均，1.1ms 转换时间，连续模式
    i2c_start(sda_pin, scl_pin);
    i2c_write_byte(sda_pin, scl_pin, INA226_ADDRESS << 1);
    i2c_write_byte(sda_pin, scl_pin, INA226_REG_CONFIG);
    i2c_write_byte(sda_pin, scl_pin, config >> 8);
    i2c_write_byte(sda_pin, scl_pin, config & 0xFF);
    i2c_stop(sda_pin, scl_pin);
    return true;
}

// 校准 INA226
bool INA226_calibrate(int sda_pin, int scl_pin, float rShuntValue, float iMaxExpected) {
    
     currentLSB = iMaxExpected/32767.0 ; // 计算电流 LSB,/INA226_REG_CALIBRATION

    uint16_t calibrationValue = (uint16_t)(0.00512 / (currentLSB * rShuntValue));

    i2c_start(sda_pin, scl_pin);
    i2c_write_byte(sda_pin, scl_pin, INA226_ADDRESS << 1);
    i2c_write_byte(sda_pin, scl_pin, INA226_REG_CALIBRATION);
    i2c_write_byte(sda_pin, scl_pin, calibrationValue >> 8);
    i2c_write_byte(sda_pin, scl_pin, calibrationValue & 0xFF);
    i2c_stop(sda_pin, scl_pin);
    return true;
}

// 读取总线电压
float INA226_readBusVoltage(int sda_pin, int scl_pin) {
    i2c_start(sda_pin, scl_pin);
    i2c_write_byte(sda_pin, scl_pin, INA226_ADDRESS << 1);
    i2c_write_byte(sda_pin, scl_pin, INA226_REG_BUSVOLTAGE);
    i2c_start(sda_pin, scl_pin);
    i2c_write_byte(sda_pin, scl_pin, (INA226_ADDRESS << 1) | 1);
    uint8_t high = i2c_read_byte(sda_pin, scl_pin, true);
    uint8_t low = i2c_read_byte(sda_pin, scl_pin, false);
    i2c_stop(sda_pin, scl_pin);
    int16_t value = (high << 8) | low;
    return value * 0.00125; // 1.25mV/LSB
}

// 读取电流
float INA226_readCurrent(int sda_pin, int scl_pin) {
    i2c_start(sda_pin, scl_pin);
    i2c_write_byte(sda_pin, scl_pin, INA226_ADDRESS << 1);
    i2c_write_byte(sda_pin, scl_pin, INA226_REG_CURRENT);
    i2c_start(sda_pin, scl_pin);
    i2c_write_byte(sda_pin, scl_pin, (INA226_ADDRESS << 1) | 1);
    uint8_t high = i2c_read_byte(sda_pin, scl_pin, true);
    uint8_t low = i2c_read_byte(sda_pin, scl_pin, false);
    i2c_stop(sda_pin, scl_pin);
    int16_t value = (high << 8) | low;
    return value * currentLSB; // 假设电流 LSB 为 1mA
}